package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.FieldsDto;
import com.qmth.distributed.print.business.bean.dto.PrintTaskDto;
import com.qmth.distributed.print.business.bean.dto.PrintTaskTotalDto;
import com.qmth.distributed.print.business.bean.dto.TaskTotalLeftDto;
import com.qmth.distributed.print.business.bean.result.ExaminationDetailResult;
import com.qmth.distributed.print.business.bean.result.ExaminationResult;
import com.qmth.distributed.print.business.bean.result.SummarizedDataResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.enums.TaskTypeEnum;
import com.qmth.distributed.print.business.mapper.ExamDetailMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.execute.AsyncCreatePdfTempleteService;
import com.qmth.distributed.print.business.util.ConvertUtil;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 考务数据 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamDetailServiceImpl extends ServiceImpl<ExamDetailMapper, ExamDetail> implements ExamDetailService {
    @Resource
    private ExamDetailCourseService examDetailCourseService;
    @Resource
    private BasicExamRuleService basicExamRuleService;
    @Resource
    private ExamDetailMapper examDetailMapper;
    @Resource
    private CacheService cacheService;
    @Resource
    private ExamStudentService examStudentService;
    @Resource
    private ExamPrintPlanService examPrintPlanService;
    @Resource
    private BasicAttachmentService basicAttachmentService;
    @Resource
    private CommonService commonService;

    @Resource
    TBTaskService tbTaskService;
    @Resource
    ConvertUtil convertUtil;

    @Resource
    AsyncCreatePdfTempleteService asyncCreatePdfTempleteService;

    private final static Logger log = LoggerFactory.getLogger(ExamDetailServiceImpl.class);

    @Transactional(rollbackFor = Exception.class)
    @Override
    public double calculateTotalSubjects(Long printPlanId) {
        List<ExamDetail> examDetailList = this.list(new QueryWrapper<ExamDetail>().lambda().eq(ExamDetail::getPrintPlanId, printPlanId));
        return examDetailList.stream().collect(Collectors.summarizingDouble(ExamDetail::getTotalSubjects)).getSum();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public double calculateTotalPackages(Long printPlanId) {
        double result = 0;
        List<ExamDetail> examDetailList = this.list(new QueryWrapper<ExamDetail>().lambda().eq(ExamDetail::getPrintPlanId, printPlanId));
        for (ExamDetail examDetail : examDetailList) {
            double count = examDetailCourseService.calculatePackagesByDetailId(examDetail.getId());
            result = result + count;
        }
        return result;
    }

    @Override
    public IPage<PrintTaskDto> listPrintTask(Long printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<PrintTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<PrintTaskDto> pirntTaskDtoIPage = this.baseMapper.listPrintTask(page, schoolId, printPlanId, status, courseCode, paperNumber, examPlace, examRoom, examStartTime, examEndTime);
        return pirntTaskDtoIPage;
    }

    @Override
    public PrintTaskTotalDto taskTotalData(Long printPlanId, String status, String courseCode, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        PrintTaskTotalDto printTaskTotalDto = this.baseMapper.taskTotalData(schoolId, printPlanId, status, courseCode, paperNumber, examPlace, examRoom, examStartTime, examEndTime);

        // 计算A3剩余
        TaskTotalLeftDto totalLeftDto = this.baseMapper.calcLeftA3(schoolId);
        if (printTaskTotalDto != null) {
            // 分别计算张数
            // 试卷
            int paperPagesA3 = totalLeftDto == null ? 0 : totalLeftDto.getPaperPagesA3();
            int printPaperCount = totalLeftDto == null ? 0 : totalLeftDto.getPrintPaperCount();
            int paperPages = paperPagesA3 % 2 == 0 ? paperPagesA3 / 2 : paperPagesA3 / 2 + 1;
            int totalPaperPages = paperPages * printPaperCount;

            // 题卡
            int cardPagesA3 = totalLeftDto == null ? 0 : totalLeftDto.getCardPagesA3();
            int printCardCount = totalLeftDto == null ? 0 : totalLeftDto.getPrintCardCount();
            int cardPages = cardPagesA3 % 2 == 0 ? cardPagesA3 / 2 : cardPagesA3 / 2 + 1;
            int totalCardPages = cardPages * printCardCount;
            printTaskTotalDto.setPagesA3Left(String.valueOf(totalPaperPages + totalCardPages));

        }
        return printTaskTotalDto;
    }

    @Override
    public String taskViewPDF(Long examDetailId) {
        ExamDetail examDetail = this.getById(examDetailId);
        if (examDetail.getAttachmentId() != null) {
            BasicAttachment attachment = basicAttachmentService.getById(examDetail.getAttachmentId());
            return attachment == null ? null : commonService.filePreview(attachment.getPath());
        }
        return null;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<FieldsDto> findExaminationFields(Long schoolId) {
        BasicExamRule basicExamRule = basicExamRuleService.list(new QueryWrapper<BasicExamRule>().lambda().eq(BasicExamRule::getSchoolId, schoolId)).get(0);
        if (basicExamRule == null) {
            throw ExceptionResultEnum.ERROR.exception("找不到该学校考务字段信息 + schoolId" + schoolId);
        }
        String requiredFields = basicExamRule.getRequiredFields();
        String extendFields = basicExamRule.getExtendFields();
        // 必选字段
        List<FieldsDto> requiredFieldsList = JSONObject.parseArray(requiredFields, FieldsDto.class);
        if (requiredFieldsList.stream().anyMatch(e -> !e.getEnable())) {
            throw ExceptionResultEnum.ERROR.exception("该学校考务字段设置存在必选字段禁用的情况 schoolId = " + schoolId);
        }
        for (FieldsDto fieldsDto : requiredFieldsList) {
            fieldsDto.setLevel("primary");
        }

        // 扩展字段
        List<FieldsDto> extendFieldsList = JSONObject.parseArray(extendFields, FieldsDto.class);
        List<String> extendCodeList = extendFieldsList.stream().map(FieldsDto::getCode).collect(Collectors.toList());
        if (extendCodeList.stream().anyMatch(e -> requiredFieldsList.stream().map(FieldsDto::getCode).collect(Collectors.toList()).contains(e))) {
            throw ExceptionResultEnum.ERROR.exception("该学校考务字段设置存在相同的code schoolId = " + schoolId);
        }
        // 有效的扩展字段
        List<FieldsDto> validExtendList = extendFieldsList.stream().filter(FieldsDto::getEnable).collect(Collectors.toList());
        for (FieldsDto fieldsDto : validExtendList) {
            fieldsDto.setLevel("secondary");
        }

        // 有效的考务字段
        List<FieldsDto> validExaminationFieldList = new ArrayList<>();
        validExaminationFieldList.addAll(requiredFieldsList);
        validExaminationFieldList.addAll(validExtendList);
        System.out.println("---" + validExaminationFieldList);

        List<String> fieldsName = validExaminationFieldList.stream().map(FieldsDto::getName).collect(Collectors.toList());
        for (String s : fieldsName) {
            System.out.println(s);
        }
        return validExaminationFieldList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void downLoadExaminationTemplate(Long schoolId, HttpServletResponse response) throws IOException {
        final String DEFALUT_EXT = ".xlsx";
        final String DEFALUT_CONTENT_TYPE = "application/vnd.ms-excel";
        List<String> fieldsNameList = this.findExaminationFields(schoolId).stream().map(FieldsDto::getName).distinct().collect(Collectors.toList());

        log.debug("导出Excel开始...");
        XSSFWorkbook wb = new XSSFWorkbook();
        String fileName = "考务数据模板";
        response.setHeader("Content-Disposition", "inline;filename=" + URLEncoder.encode(fileName, SystemConstant.CHARSET_NAME) + DEFALUT_EXT);
        response.setContentType(DEFALUT_CONTENT_TYPE);
        XSSFSheet sheet = wb.createSheet(cacheService.schoolCache(schoolId).getName() + "考务数据模板");

        XSSFFont font = wb.createFont();
        font.setFontHeightInPoints((short) 11);
        font.setFontName("宋体"); //什么字体

        XSSFRow row = sheet.createRow(0);
        XSSFCellStyle style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setFont(font);

        for (int i = 0; i < fieldsNameList.size(); i++) {
            XSSFCell cell = row.createCell(i);
            cell.setCellValue(fieldsNameList.get(i));
            cell.setCellStyle(style);
        }
        ServletOutputStream outputStream = response.getOutputStream();
        wb.write(outputStream);
        outputStream.flush();
        outputStream.close();
        log.debug("导出Excel结束...");

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public IPage<ExaminationResult> findExaminationBriefPage(Long schoolId, Long printPlanId, String courseCode, String paperNumber, String examPlace, String examRoom, String packageCode, int pageNumber, int pageSize) {
        IPage<ExaminationResult> page = examDetailMapper.findBriefPage(new Page<>(pageNumber, pageSize), schoolId, printPlanId, courseCode, paperNumber, examPlace, examRoom, packageCode);
        List<ExaminationResult> list = page.getRecords();
        for (ExaminationResult examinationResult : list) {
            String examDetailCourseIds = examinationResult.getExamDetailCourseIds();
            if (examDetailCourseIds == null || examDetailCourseIds.length() == 0) {
                throw ExceptionResultEnum.ERROR.exception("考务数据确实课程信息");
            }
            String[] arr = examDetailCourseIds.split(",");

            final String separator = "、";
            String courseNameCodeStr = "";
            String paperNumberStr = "";
            for (String examDetailCourseId : arr) {
                ExamDetailCourse examDetailCourse = examDetailCourseService.getById(examDetailCourseId);
                String code = examDetailCourse.getCourseCode();
                String name = examDetailCourse.getCourseName();
                String number = examDetailCourse.getPaperNumber();
                courseNameCodeStr = courseNameCodeStr + name + "(" + code + ")" + separator;
                paperNumberStr = paperNumberStr + number + separator;
            }
            courseNameCodeStr = courseNameCodeStr.substring(0, courseNameCodeStr.length() - separator.length());
            paperNumberStr = paperNumberStr.substring(0, paperNumberStr.length() - separator.length());
            examinationResult.setCourseNameCode(courseNameCodeStr);
            examinationResult.setPaperNumber(paperNumberStr);
        }
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public SummarizedDataResult findSummarizedData(Long schoolId, Long printPlanId, String courseCode, String paperNumber, String examPlace, String examRoom, String packageCode) {
        IPage<ExaminationResult> page = this.findExaminationBriefPage(schoolId, printPlanId, courseCode, paperNumber, examPlace, examRoom, packageCode, SystemConstant.PAGE_NUMBER, SystemConstant.PAGE_SIZE);
        List<ExaminationResult> list = page.getRecords();
        int totalSubjects = 0;
        int packageCount = 0;
        for (ExaminationResult examinationResult : list) {
            int subjects = examinationResult.getTotalSubjects();
            int packages = examinationResult.getCourseNameCode().split("、").length;
            totalSubjects = totalSubjects + subjects;
            packageCount = packageCount + packages;
        }
        SummarizedDataResult summarizedDataResult = new SummarizedDataResult();
        summarizedDataResult.setSchoolId(schoolId);
        summarizedDataResult.setTotalSubjects(totalSubjects);
        summarizedDataResult.setPackageCount(packageCount);
        return summarizedDataResult;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public IPage<ExaminationDetailResult> findExaminationDetail(Long printPlanId, String courseCode, String paperNumber, String examPlace, String examRoom, String studentParam, int pageNumber, int pageSize) {
        return examDetailMapper.findDetailPage(new Page<>(pageNumber, pageSize), printPlanId, courseCode, paperNumber, examPlace, examRoom, studentParam);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public IPage<ExaminationDetailResult> findExaminationDetail(Long examDetailId, int pageNumber, int pageSize) {
        return examDetailMapper.findDetailPageById(new Page<>(pageNumber, pageSize), examDetailId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<String> findExamPlaceDatasource() {
        List<ExamDetail> examDetail = this.list();
        return examDetail.stream().map(ExamDetail::getExamPlace).distinct().collect(Collectors.toList());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<String> findExamRoomDatasource() {
        List<ExamDetail> examDetail = this.list();
        return examDetail.stream().map(ExamDetail::getExamRoom).distinct().collect(Collectors.toList());
    }

    @Override
    public boolean submitTask(ExamDetail examDetail) {
        // 检查前置操作是否完成 todo
        UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetail::getStatus, ExamDetailStatusEnum.WAITING).eq(ExamDetail::getId, examDetail.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean taskCancel(ExamDetail examDetail) {
        ExamDetail detail = this.getById(examDetail.getId());
        if (!detail.getStatus().name().equals(ExamDetailStatusEnum.WAITING.name())) {
            throw ExceptionResultEnum.ERROR.exception("印刷状态为“待印刷”的任务可撤回提交");
        }
        UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetail::getStatus, ExamDetailStatusEnum.READY).eq(ExamDetail::getId, examDetail.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean resubmitTask(ExamDetail examDetail) throws IOException {
        ExamDetail detail = this.getById(examDetail.getId());
        if (!detail.getStatus().name().equals(ExamDetailStatusEnum.PRINTING.name())
                && !detail.getStatus().name().equals(ExamDetailStatusEnum.FINISH.name())) {
            throw ExceptionResultEnum.ERROR.exception("印刷状态为“印刷中”、“已完成”的任务可重新提交印刷");
        }
        // 调用生成pdf方法
        Map<String, Object> map = tbTaskService.saveTask(TaskTypeEnum.CREATE_PDF, detail.getPrintPlanId());
        if (Objects.nonNull(examDetail.getId())) {
            map.computeIfAbsent("examDetailId", v -> detail.getId());
        }
        asyncCreatePdfTempleteService.createPdf(map);
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void disposeExamDetailByExaminationExcel(List<Map<String, Object>> dataList, Long userId) {
        List<Map<String, Object>> examDetailKeyList = dataList.stream().flatMap(e -> {
            Map<String, Object> map = new HashMap<>();
            map.put("schoolId", e.get("schoolId"));
            map.put("printPlanId", e.get("printPlanId"));
            map.put("printPlanName", e.get("printPlanName"));
            map.put("examPlace", e.get("examPlace"));
            map.put("examRoom", e.get("examRoom"));
            map.put("examStartTime", e.get("examStartTime"));
            map.put("examEndTime", e.get("examEndTime"));
            return Stream.of(map);
        }).distinct().collect(Collectors.toList());

        System.out.println("examDetailKeyList = " + JSON.toJSONString(examDetailKeyList));

        List<ExamDetail> examDetailList = new ArrayList<>();
        for (Map<String, Object> map : examDetailKeyList) {
            String examPlace = String.valueOf(map.get("examPlace"));
            String examRoom = String.valueOf(map.get("examRoom"));
            String examStartTime = String.valueOf(map.get("examStartTime"));
            String examEndTime = String.valueOf(map.get("examEndTime"));
            System.out.println("dataList : " + JSON.toJSONString(dataList));
            System.out.println(examPlace + examRoom + examStartTime + examEndTime);
            long totalSubjects = dataList.stream()
                    .filter(e -> String.valueOf(e.get("examPlace")).equals(examPlace) &&
                            String.valueOf(e.get("examRoom")).equals(examRoom) &&
                            String.valueOf(e.get("examStartTime")).equals(examStartTime) &&
                            String.valueOf(e.get("examEndTime")).equals(examEndTime)).count();

            System.out.println("totalSubjects" + totalSubjects);
            ExamDetail examDetail = new ExamDetail();
            examDetail.setId(SystemConstant.getDbUuid());
            examDetail.setPackageCode(convertUtil.getIncre("p-","packageCode"));
            examDetail.setSchoolId(Long.valueOf(String.valueOf(map.get("schoolId"))));
            examDetail.setPrintPlanId(Long.valueOf(String.valueOf(map.get("printPlanId"))));
            examDetail.setPrintPlanName(String.valueOf(map.get("printPlanName")));
            examDetail.setExamPlace(examPlace);
            examDetail.setExamRoom(examRoom);
            examDetail.setStatus(ExamDetailStatusEnum.NEW);
            examDetail.setTotalSubjects(Math.toIntExact(totalSubjects));
            examDetail.setExamStartTime(SystemConstant.convertIdToLong(examStartTime));
            examDetail.setExamEndTime(SystemConstant.convertIdToLong(examEndTime));
            examDetail.setCreateId(userId);
            examDetail.setUpdateId(userId);
            examDetailList.add(examDetail);
        }
        this.saveBatch(examDetailList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void disposeExamDetailCourseByExaminationExcel(List<Map<String, Object>> dataList, Long userId) {

        List<Map<String, Object>> examDetailKeyList = dataList.stream().flatMap(e -> {
            Map<String, Object> map = new HashMap<>();
            map.put("schoolId", e.get("schoolId"));
            map.put("printPlanId", e.get("printPlanId"));
            map.put("examPlace", e.get("examPlace"));
            map.put("examRoom", e.get("examRoom"));
            map.put("examStartTime", e.get("examStartTime"));
            map.put("examEndTime", e.get("examEndTime"));
            map.put("courseCode", e.get("courseCode"));
            map.put("courseName", e.get("courseName"));
            map.put("paperNumber", e.get("paperNumber"));
            return Stream.of(map);
        }).distinct().collect(Collectors.toList());

        List<ExamDetailCourse> examDetailCourseList = new ArrayList<>();
        for (Map<String, Object> map : examDetailKeyList) {
            Long schoolId = Long.valueOf(String.valueOf(map.get("schoolId")));
            Long printPlanId = Long.valueOf(String.valueOf(map.get("printPlanId")));
            String examPlace = String.valueOf(map.get("examPlace"));
            String examRoom = String.valueOf(map.get("examRoom"));
            Long examStartTime = SystemConstant.convertIdToLong(String.valueOf(map.get("examStartTime")));
            Long examEndTime = SystemConstant.convertIdToLong(String.valueOf(map.get("examEndTime")));
            String courseCode = String.valueOf(map.get("courseCode"));
            String courseName = String.valueOf(map.get("courseName"));
            String paperNumber = String.valueOf(map.get("paperNumber"));
            List<ExamDetail> examDetailList = this.list(new QueryWrapper<ExamDetail>().lambda()
                    .eq(ExamDetail::getPrintPlanId, printPlanId)
                    .eq(ExamDetail::getExamPlace, examPlace)
                    .eq(ExamDetail::getExamRoom, examRoom)
                    .eq(ExamDetail::getExamStartTime, examStartTime)
                    .eq(ExamDetail::getExamEndTime, examEndTime));
            if (examDetailList.size() != 1) {
                throw ExceptionResultEnum.ERROR.exception("无法找到新增的考务场次数据");
            }
            Long examDetailId = examDetailList.get(0).getId();
            ExamDetailCourse examDetailCourse = new ExamDetailCourse();
            examDetailCourse.setId(SystemConstant.getDbUuid());
            examDetailCourse.setSchoolId(schoolId);
            examDetailCourse.setExamDetailId(examDetailId);
            examDetailCourse.setCourseCode(courseCode);
            examDetailCourse.setCourseName(courseName);
            examDetailCourse.setPaperNumber(paperNumber);
            examDetailCourse.setCreateId(userId);
            examDetailCourse.setUpdateId(userId);
            examDetailCourseList.add(examDetailCourse);
        }
        examDetailCourseService.saveBatch(examDetailCourseList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void disposeExamStudentByExaminationExcel(List<Map<String, Object>> dataList, Long userId) {
        List<ExamStudent> examStudentList = new ArrayList<>();
        for (Map<String, Object> map : dataList) {
            Long schoolId = Long.valueOf(String.valueOf(map.get("schoolId")));
            Long printPlanId = Long.valueOf(String.valueOf(map.get("printPlanId")));
            String examPlace = String.valueOf(map.get("examPlace"));
            String examRoom = String.valueOf(map.get("examRoom"));
            Long examStartTime = SystemConstant.convertIdToLong(String.valueOf(map.get("examStartTime")));
            Long examEndTime = SystemConstant.convertIdToLong(String.valueOf(map.get("examEndTime")));
            String courseCode = String.valueOf(map.get("courseCode"));
            String courseName = String.valueOf(map.get("courseName"));
            String paperNumber = String.valueOf(map.get("paperNumber"));
            String studentName = String.valueOf(map.get("studentName"));
            String studentCode = String.valueOf(map.get("studentCode"));
            List<ExamDetail> examDetailList = this.list(new QueryWrapper<ExamDetail>().lambda()
                    .eq(ExamDetail::getPrintPlanId, printPlanId)
                    .eq(ExamDetail::getExamPlace, examPlace)
                    .eq(ExamDetail::getExamRoom, examRoom)
                    .eq(ExamDetail::getExamStartTime, examStartTime)
                    .eq(ExamDetail::getExamEndTime, examEndTime));
            if (examDetailList.size() != 1) {
                throw ExceptionResultEnum.ERROR.exception("无法找到新增的考务场次数据");
            }
            Long examDetailId = examDetailList.get(0).getId();
            List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.list(new QueryWrapper<ExamDetailCourse>().lambda()
                    .eq(ExamDetailCourse::getExamDetailId, examDetailId)
                    .eq(ExamDetailCourse::getCourseCode, courseCode)
                    .eq(ExamDetailCourse::getCourseName, courseName)
                    .eq(ExamDetailCourse::getPaperNumber, paperNumber));
            if (examDetailCourseList.size() != 1) {
                throw ExceptionResultEnum.ERROR.exception("无法找到新增的考务科目数据");
            }
            Long examDetailCourseId = examDetailCourseList.get(0).getId();

            ExamStudent examStudent = new ExamStudent();
            examStudent.setId(SystemConstant.getDbUuid());
            examStudent.setSchoolId(schoolId);
            examStudent.setExamDetailCourseId(examDetailCourseId);
            examStudent.setStudentName(studentName);
            examStudent.setStudentCode(studentCode);
            examStudent.setTicketNumber(studentCode);
            examStudent.setSiteNumber("");
            examStudent.setCreateId(userId);
            examStudent.setUpdateId(userId);
            examStudentList.add(examStudent);
        }
        examStudentService.saveBatch(examStudentList);
    }

    @Override
    public List<ExamDetail> listByCourseCodeAndPaperNumber(Long schoolId, String courseCode, String paperNumber) {
        QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
        examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getSchoolId, schoolId)
                .eq(ExamDetailCourse::getCourseCode, courseCode)
                .eq(ExamDetailCourse::getPaperNumber, paperNumber);
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.list(examDetailCourseQueryWrapper);
        if (examDetailCourses != null && examDetailCourses.size() > 0) {
            Set<Long> examDetailIds = examDetailCourses.stream().map(m -> m.getExamDetailId()).collect(Collectors.toSet());
            List<ExamDetail> examDetails = this.listByIds(examDetailIds);
            return examDetails;
        }
        return null;
    }

    @Override
    public List<ExamDetail> listByExamPrintId(Long printPlanId) {
        QueryWrapper<ExamDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetail::getPrintPlanId, printPlanId)
                .ne(ExamDetail::getStatus, ExamDetailStatusEnum.READY);
        return this.list(queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deleteExaminationData(Long printPlanId) {
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(printPlanId);
        if (examPrintPlan.getId() == null || examPrintPlan.getId() <= 0) {
            throw ExceptionResultEnum.ERROR.exception("未找主键为 '" + printPlanId + "' 的印刷计划信息");
        }
        PrintPlanStatusEnum status = examPrintPlan.getStatus();
        if (PrintPlanStatusEnum.NEW != status && PrintPlanStatusEnum.READY != status) {
            throw ExceptionResultEnum.ERROR.exception(status + "状态下的印刷计划不能删除考务数据");
        }
        // 准备删除的考务-场次表id
        List<Long> examDetailIds = this.list(new QueryWrapper<ExamDetail>().lambda().eq(ExamDetail::getPrintPlanId, printPlanId))
                .stream().map(ExamDetail::getId).collect(Collectors.toList());
        if (examDetailIds.size() == 0){
            return;
        }
        // 删除exam_detail 表数据
        this.remove(new QueryWrapper<ExamDetail>().lambda().in(ExamDetail::getId, examDetailIds));

        // 准备删除的考务-科目表id
        List<Long> examDetailCourseIds = examDetailCourseService.list(new QueryWrapper<ExamDetailCourse>().lambda().in(ExamDetailCourse::getExamDetailId, examDetailIds))
                .stream().map(ExamDetailCourse::getId).collect(Collectors.toList());
        if (examDetailCourseIds.size() == 0){
            return;
        }
        // 删除exam_detail_course 表数据
        examDetailCourseService.remove(new QueryWrapper<ExamDetailCourse>().lambda().in(ExamDetailCourse::getId, examDetailCourseIds));

        // 准备删除的考务-考生表id
        List<Long> examStudentIds = examStudentService.list(new QueryWrapper<ExamStudent>().lambda().in(ExamStudent::getExamDetailCourseId, examDetailCourseIds))
                .stream().map(ExamStudent::getId).collect(Collectors.toList());
        if (examStudentIds.size() == 0){
            return;
        }
        // 删除exam_student 表数据
        examStudentService.remove(new QueryWrapper<ExamStudent>().lambda().in(ExamStudent::getId, examStudentIds));
    }
}
